﻿using Microsoft.Extensions.Logging;
using SlnMesnac.Common;
using SlnMesnac.Model.domain;
using SlnMesnac.Repository.service.@base;
using System;
using System.Collections.Generic;
using System.ComponentModel.Design;
using System.Linq;
using System.Linq.Expressions;
using System.Text;

namespace SlnMesnac.Repository.service.Impl
{
    public class ProdPlanInfoServiceImpl : BaseServiceImpl<ProdPLanInfo>, ProdPlanInfoService
    {
        private ILogger<ProdPlanInfoServiceImpl> _logger;
        public ProdPlanInfoServiceImpl(Repository<ProdPLanInfo> repository, ILogger<ProdPlanInfoServiceImpl> logger) : base(repository)
        {
            _logger = logger;
        }

        /// <summary>
        /// 通过订单编号、工单编号、物料码获取工单信息
        /// </summary>
        /// <param name="orderCode"></param>
        /// <param name="planCode"></param>
        /// <param name="materialCode"></param>
        /// <returns></returns>
        public List<ProdPLanInfo> GetRecordStaffAttendancesByConditions(string? orderCode, string? planCode, string? materialCode, string? stationCode, string? planStatus)
        {
            List<ProdPLanInfo> recordStaffAttendances = new List<ProdPLanInfo>();
            List<ProdPLanInfo> planInfoList = _rep.AsQueryable().WhereIF(!string.IsNullOrEmpty(orderCode), x => x.OrderCode == orderCode)
                .WhereIF(!string.IsNullOrEmpty(planCode), x => x.PlanCode == planCode)
                .WhereIF(!string.IsNullOrEmpty(materialCode), x => x.MaterialCode == materialCode)
                .WhereIF(!string.IsNullOrEmpty(stationCode), x => x.StationCode == stationCode)
                .WhereIF(!string.IsNullOrEmpty(planStatus), x => x.PlanStatus == planStatus || x.PlanStatus == "4")
                .OrderByDescending(x => x.ObjId)
                .ToList();
            return planInfoList;
        }

        /// <summary>
        /// 通过订单编号、工单编号、物料码获取工单信息
        /// </summary>
        /// <param name="orderCode"></param>
        /// <param name="planCode"></param>
        /// <param name="materialCode"></param>
        /// <returns></returns>
        public List<ProdPLanInfo> GetPlanInfoByConditions(string? orderCode, string? planCode, string? materialCode, string? stationCode, string? planStatus)
        {
            List<ProdPLanInfo> recordStaffAttendances = new List<ProdPLanInfo>();
            List<ProdPLanInfo> planInfoList = _rep.AsQueryable().WhereIF(!string.IsNullOrEmpty(orderCode), x => x.OrderCode == orderCode)
                .WhereIF(!string.IsNullOrEmpty(planCode), x => x.PlanCode == planCode)
                .WhereIF(!string.IsNullOrEmpty(materialCode), x => x.MaterialCode == materialCode)
                .WhereIF(!string.IsNullOrEmpty(stationCode), x => x.StationCode == stationCode)
                .WhereIF(!string.IsNullOrEmpty(planStatus), x => x.PlanStatus == planStatus)
                .OrderByDescending(x => x.ObjId)
                .ToList();
            return planInfoList;
        }

        public List<ProdPLanInfo> GetRecordStaffAttendances()
        {
            List<ProdPLanInfo> pLanInfos = null;
            try
            {
                pLanInfos = base._rep.GetList();
                pLanInfos.Reverse();
            }
            catch (Exception ex)
            {
                _logger.LogError($"获取员工打卡信息异常{ex.Message}");
            }
            return pLanInfos;
        }

        /// <summary>
        /// 根据工单号获取此工单
        /// </summary>
        /// <param name="planCode"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        public ProdPLanInfo GetProdPLanInfoByOrderCode(string stationCode, string orderCode)
        {
            ProdPLanInfo prodPLan = new ProdPLanInfo();
            try
            {//只显示未执行的
                prodPLan = _rep.GetFirst(x => x.OrderCode == orderCode && x.StationCode == stationCode);
                //&& (x.PlanStatus == "0" || x.PlanStatus == "4")
            }
            catch (Exception ex)
            {
                _logger.LogError($"根据工单号获取此工单异常{ex.Message}");
            }
            return prodPLan;
        }

        /// <summary>
        /// 更新工单状态
        /// </summary>
        /// <param name="orderCode"></param>
        /// <param name="stationCode"></param>
        /// <param name="deviceCode"></param>
        /// <param name="processCode"></param>
        /// <param name="status"></param>
        /// <exception cref="NotImplementedException"></exception>
        public bool UpdatePlanStatus(string orderCode, string stationCode, string deviceCode, string processCode, string status)
        {
            bool result = false;
            try
            {
                var planInfo = _rep.GetFirst(x => x.OrderCode == orderCode && x.StationCode == stationCode && x.DeviceCode == deviceCode && x.ProcessCode == processCode && x.PlanStatus == "0");
                if (planInfo != null)
                {
                    planInfo.PlanStatus = status;
                    result = _rep.Update(planInfo);
                }
            }
            catch (Exception ex)
            {
                _logger.LogError($"修改计划状态异常:{ex.Message}");
            }
            return result;
        }
    }
}
